@@ -1,13 +1,15 @@
 from __future__ import annotations
 
 import asyncio
+import contextvars
 import http
 import logging
 import re
 import urllib
 from asyncio.events import TimerHandle
 from collections import deque
-from typing import Any, Callable, Literal, cast
+from collections.abc import Callable
+from typing import Any, Literal, cast
 
 import httptools
 
@@ -58,6 +60,14 @@ class HttpToolsProtocol(asyncio.Protocol):
         self.access_logger = logging.getLogger("uvicorn.access")
         self.access_log = self.access_logger.hasHandlers()
         self.parser = httptools.HttpRequestParser(self)
+
+        try:
+            # Enable dangerous leniencies to allow server to a response on the first request from a pipelined request.
+            self.parser.set_dangerous_leniencies(lenient_data_after_close=True)
+        except AttributeError:  # pragma: no cover
+            # httptools < 0.6.3
+            pass
+
         self.ws_protocol_class = config.ws_protocol_class
         self.root_path = config.root_path
         self.limit_concurrency = config.limit_concurrency
@@ -214,7 +224,7 @@ class HttpToolsProtocol(asyncio.Protocol):
         self.headers = []
         self.scope = {  # type: ignore[typeddict-item]
             "type": "http",
-            "asgi": {"version": self.config.asgi_version, "spec_version": "2.4"},
+            "asgi": {"version": self.config.asgi_version, "spec_version": "2.3"},
             "http_version": "1.1",
             "server": self.server,
             "client": self.client,
@@ -279,7 +289,12 @@ class HttpToolsProtocol(asyncio.Protocol):
         )
         if existing_cycle is None or existing_cycle.response_complete:
             # Standard case - start processing the request.
-            task = self.loop.create_task(self.cycle.run_asgi(app))
+            # For the asyncio loop, we need to explicitly start with an empty context
+            # as it can be polluted from previous ASGI runs.
+            # See https://github.com/python/cpython/issues/140947 for details.
+            task = contextvars.Context().run(self.loop.create_task, self.cycle.run_asgi(app))
+            # TODO: Replace the line above with the line below for Python >= 3.11
+            # task = self.loop.create_task(self.cycle.run_asgi(app), context=contextvars.Context())
             task.add_done_callback(self.tasks.discard)
             self.tasks.add(task)
         else:
